@@ -32,7 +32,6 @@ further details.
 # - verify that pre-created database is owned by "gm-dbo"
 # - rework under assumption that there is only one DB
 #==================================================================
-__version__ = "$Revision: 1.113 $"
 __author__ = "Karsten.Hilbert@gmx.net"
 __license__ = "GPL"
 
@@ -691,6 +690,12 @@ class database:
 		curs.execute("alter database %s set default_transaction_read_only to on" % self.name)
 		# we want checking of function bodies
 		curs.execute("alter database %s set check_function_bodies to on" % self.name)
+		# we want checking of data checksums if available
+		# remove exception handler when 9.3 is default
+		try:
+			curs.execute("alter database %s set ignore_checksum_failure to off" % self.name)
+		except:
+			_log.exception('PostgreSQL version < 9.3 does not support <ignore_checksum_failure>')
 		curs.close()
 
 		self.conn.commit()
@@ -1165,7 +1170,33 @@ class database:
 				}
 				gmPG2.run_rw_queries(link_obj = self.conn, queries = [{'cmd': cmd}])
 
-		print_msg("==> setting up notifications ...")
+		# always re-create generic super signal (if exists)
+		print_msg("==> setting up generic notifications ...")
+		_log.debug('creating generic modification announcement triggers on all registered tables')
+		curs = self.conn.cursor()
+		cmd = u"""
+			SELECT EXISTS (
+				SELECT 1 FROM information_schema.routines WHERE
+					routine_name = 'create_all_table_mod_triggers'
+						AND
+					routine_schema = 'gm'
+			)"""
+		curs.execute(cmd)
+		result = curs.fetchone()
+		if result[0] is True:
+			_log.debug('creating generic modification announcement triggers on registered tables')
+			cmd = u"SELECT gm.create_all_table_mod_triggers(True::boolean)"
+			curs.execute(cmd)
+			result = curs.fetchone()
+			curs.close()
+			if result[0] is False:
+				_log.error('cannot create generic modification announcement triggers on all tables')
+				return None
+		else:
+			curs.close()
+			_log.debug('NOT creating generic modification announcement triggers, functionality not available')
+
+		print_msg("==> setting up (old style) notifications ...")
 		# get configuration
 		tmp = cfg_get(self.section, 'notification disable')
 		# if this option is not given, assume we want notification
@@ -1251,6 +1282,8 @@ class gmBundle:
 			_log.error("Cannot load minimum required PostgreSQL version from config file.")
 			return None
 
+		_log.info("minimum required PostgreSQL version: %s" % required_version)
+
 		converted, pg_ver = gmTools.input2decimal(gmPG2.postgresql_version)
 		if not converted:
 			_log.error('error checking PostgreSQL version')
@@ -1264,7 +1297,7 @@ class gmBundle:
 			_log.error("Reported live PostgreSQL version [%s] is smaller than the required minimum version [%s]." % (gmPG2.postgresql_version, required_version))
 			return None
 
-		_log.info("installed PostgreSQL version: [%s] - this is fine with me" % gmPG2.postgresql_version)
+		_log.info("installed PostgreSQL version: %s - this is fine with me" % gmPG2.postgresql_version)
 		return True
 #==================================================================
 def bootstrap_bundles():
@@ -1573,6 +1606,7 @@ def main():
 
 	_log.info("shutdown")
 	print("Done bootstrapping GNUmed database: We very likely succeeded.")
+	print 'log:', gmLog2._logfile_name
 
 #==================================================================
 if __name__ == "__main__":
@@ -1580,7 +1614,7 @@ if __name__ == "__main__":
 	gmI18N.activate_locale()
 	gmLog2.set_string_encoding()
 
-	_log.info("startup (%s)" % __version__)
+	_log.info("startup")
 
 	try:
 		main()
