@@ -1,14 +1,17 @@
-# -*- coding: utf8 -*-
+# -*- coding: utf-8 -*-
 __doc__ = """GNUmed general tools."""
 
 #===========================================================================
 __author__ = "K. Hilbert <Karsten.Hilbert@gmx.net>"
-__license__ = "GPL (details at http://www.gnu.org)"
+__license__ = "GPL v2 or later (details at http://www.gnu.org)"
 
 # std libs
 import re as regex, sys, os, os.path, csv, tempfile, logging, hashlib
+import platform
+import subprocess
 import decimal
 import cPickle, zlib
+import xml.sax.saxutils as xml_tools
 
 
 # GNUmed libs
@@ -35,6 +38,9 @@ _log = logging.getLogger('gm.tools')
 ) = range(6)
 
 
+u_currency_pound = u'\u00A3'				# Pound sign
+u_currency_sign = u'\u00A4'					# generic currency sign
+u_currency_yen = u'\u00A5'					# Yen sign
 u_right_double_angle_quote = u'\u00AB'		# <<
 u_registered_trademark = u'\u00AE'
 u_plus_minus = u'\u00B1'
@@ -42,11 +48,21 @@ u_left_double_angle_quote = u'\u00BB'		# >>
 u_one_quarter = u'\u00BC'
 u_one_half = u'\u00BD'
 u_three_quarters = u'\u00BE'
-u_ellipsis = u'\u2026'
+u_multiply = u'\u00D7'						# x
+u_greek_ALPHA = u'\u0391'
+u_greek_alpha = u'\u03b1'
+u_greek_OMEGA = u'\u03A9'
+u_greek_omega = u'\u03c9'
+u_triangular_bullet = u'\u2023'				# triangular bullet  (>)
+u_ellipsis = u'\u2026'						# ...
+u_euro = u'\u20AC'							# EURO sign
+u_numero = u'\u2116'						# No. / # sign
 u_down_left_arrow = u'\u21B5'				# <-'
 u_left_arrow = u'\u2190'					# <--
 u_right_arrow = u'\u2192'					# -->
-u_sum = u'\u2211'
+u_left_arrow_with_tail = u'\u21a2'			# <--<
+u_sum = u'\u2211'							# sigma
+u_almost_equal_to = u'\u2248'				# approximately / nearly / roughly
 u_corresponds_to = u'\u2258'
 u_infinity = u'\u221E'
 u_diameter = u'\u2300'
@@ -61,6 +77,7 @@ u_box_bottom_right_arc = u'\u256f'
 u_box_bottom_left_arc = u'\u2570'
 u_box_horiz_light_heavy = u'\u257c'
 u_box_horiz_heavy_light = u'\u257e'
+u_skull_and_crossbones = u'\u2620'
 u_frowning_face = u'\u2639'
 u_smiling_face = u'\u263a'
 u_black_heart = u'\u2665'
@@ -71,8 +88,9 @@ u_pencil_1 = u'\u270e'
 u_pencil_2 = u'\u270f'
 u_pencil_3 = u'\u2710'
 u_latin_cross = u'\u271d'
+u_kanji_yen = u'\u5186'						# Yen kanji
 u_replacement_character = u'\ufffd'
-
+u_link_symbol = u'\u1f517'
 
 #===========================================================================
 def handle_uncaught_exception_console(t, v, tb):
@@ -105,7 +123,7 @@ class gmPaths(gmBorg.cBorg):
 	.user_config_dir
 	.system_config_dir
 	.system_app_data_dir
-	.tmp_dir (only readable)
+	.tmp_dir (readonly)
 	"""
 	def __init__(self, app_name=None, wx=None):
 		"""Setup pathes.
@@ -139,15 +157,25 @@ class gmPaths(gmBorg.cBorg):
 		self.__home_dir = None
 
 		# where the main script (the "binary") is installed
-		#self.local_base_dir = os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), '..', '.'))
-		self.local_base_dir = os.path.abspath(os.path.dirname(sys.argv[0]))
+		if getattr(sys, 'frozen', False):
+			_log.info('frozen app, installed into temporary path')
+			# this would find the path of *THIS* file
+			#self.local_base_dir = os.path.dirname(__file__)
+			# while this is documented on the web, the ${_MEIPASS2} does not exist
+			#self.local_base_dir = os.environ.get('_MEIPASS2')
+			# this is what Martin Zibricky <mzibr.public@gmail.com> told us to use
+			# when asking about this on pyinstaller@googlegroups.com
+			#self.local_base_dir = sys._MEIPASS
+			# however, we are --onedir, so we should look at sys.executable
+			# as per the pyinstaller manual
+			self.local_base_dir = os.path.dirname(sys.executable)
+		else:
+			self.local_base_dir = os.path.abspath(os.path.dirname(sys.argv[0]))
 
 		# the current working dir at the OS
 		self.working_dir = os.path.abspath(os.curdir)
 
 		# user-specific config dir, usually below the home dir
-		#mkdir(os.path.expanduser(os.path.join('~', '.%s' % app_name)))
-		#self.user_config_dir = os.path.expanduser(os.path.join('~', '.%s' % app_name))
 		mkdir(os.path.join(self.home_dir, '.%s' % app_name))
 		self.user_config_dir = os.path.join(self.home_dir, '.%s' % app_name)
 
@@ -216,9 +244,12 @@ class gmPaths(gmBorg.cBorg):
 	#--------------------------------------
 	def __log_paths(self):
 		_log.debug('sys.argv[0]: %s', sys.argv[0])
+		_log.debug('sys.executable: %s', sys.executable)
+		_log.debug('sys._MEIPASS: %s', getattr(sys, '_MEIPASS', '<not found>'))
+		_log.debug('os.environ["_MEIPASS2"]: %s', os.environ.get('_MEIPASS2', '<not found>'))
+		_log.debug('__file__ : %s', __file__)
 		_log.debug('local application base dir: %s', self.local_base_dir)
 		_log.debug('current working dir: %s', self.working_dir)
-		#_log.debug('user home dir: %s', os.path.expanduser('~'))
 		_log.debug('user home dir: %s', self.home_dir)
 		_log.debug('user-specific config dir: %s', self.user_config_dir)
 		_log.debug('system-wide config dir: %s', self.system_config_dir)
@@ -311,8 +342,44 @@ class gmPaths(gmBorg.cBorg):
 #===========================================================================
 # file related tools
 #---------------------------------------------------------------------------
+def gpg_decrypt_file(filename=None, passphrase=None):
+
+	if platform.system() == 'Windows':
+		exec_name = 'gpg.exe'
+	else:
+		exec_name = 'gpg'
+
+	tmp, fname = os.path.split(filename)
+	basename, tmp = os.path.splitext(fname)
+	filename_decrypted = get_unique_filename(prefix = '%s-decrypted-' % basename)
+
+	args = [exec_name, '--verbose', '--batch', '--yes', '--passphrase-fd', '0', '--output', filename_decrypted, '--decrypt', filename]
+	_log.debug('GnuPG args: %s' % str(args))
+
+	try:
+		gpg = subprocess.Popen (
+			args = args,
+			stdin = subprocess.PIPE,
+			stdout = subprocess.PIPE,
+			stderr = subprocess.PIPE,
+			close_fds = False
+		)
+	except (OSError, ValueError, subprocess.CalledProcessError):
+		_log.exception('there was a problem executing gpg')
+		gmDispatcher.send(signal = u'statustext', msg = _('Error running GnuPG. Cannot decrypt data.'), beep = True)
+		return
+
+	out, error = gpg.communicate(passphrase)
+	_log.debug('gpg returned [%s]', gpg.returncode)
+	if gpg.returncode != 0:
+		_log.debug('GnuPG STDOUT:\n%s', out)
+		_log.debug('GnuPG STDERR:\n%s', error)
+		return None
+
+	return filename_decrypted
+#---------------------------------------------------------------------------
 def file2md5(filename=None, return_hex=True):
-	blocksize = 2**10 * 128			# 128k, since md5 use 128 byte blocks
+	blocksize = 2**10 * 128			# 128k, since md5 uses 128 byte blocks
 	_log.debug('md5(%s): <%s> byte blocks', filename, blocksize)
 
 	f = open(filename, 'rb')
@@ -372,12 +439,17 @@ def unicode_csv_reader(unicode_csv_data, dialect=csv.excel, encoding='utf-8', **
 		else:
 			yield [ unicode(cell, encoding) for cell in row ]
 			#yield [unicode(cell, 'utf-8') for cell in row]
+
+#---------------------------------------------------------------------------
+def fname_stem(filename):
+	return os.path.splitext(os.path.basename(filename))[0]
+
 #---------------------------------------------------------------------------
 def get_unique_filename(prefix=None, suffix=None, tmp_dir=None):
 	"""This introduces a race condition between the file.close() and
 	actually using the filename.
 
-	The file will not exist after calling this function.
+	The file will NOT exist after calling this function.
 	"""
 	if tmp_dir is not None:
 		if (
@@ -385,7 +457,7 @@ def get_unique_filename(prefix=None, suffix=None, tmp_dir=None):
 				or
 			not os.access(tmp_dir, os.X_OK | os.W_OK)
 		):
-			_log.info('cannot find temporary dir [%s], using system default', tmp_dir)
+			_log.warning('cannot find temporary dir [%s], using system default', tmp_dir)
 			tmp_dir = None
 
 	kwargs = {'dir': tmp_dir}
@@ -411,6 +483,8 @@ def get_unique_filename(prefix=None, suffix=None, tmp_dir=None):
 def import_module_from_directory(module_path=None, module_name=None, always_remove_path=False):
 	"""Import a module from any location."""
 
+	_log.debug('CWD: %s', os.getcwd())
+
 	remove_path = always_remove_path or False
 	if module_path not in sys.path:
 		_log.info('appending to sys.path: [%s]' % module_path)
@@ -445,7 +519,7 @@ _GB = 1024 * _MB
 _TB = 1024 * _GB
 _PB = 1024 * _TB
 #---------------------------------------------------------------------------
-def size2str(size=0, template='%s'):
+def size2str(size=0, template=u'%s'):
 	if size == 1:
 		return template % _('1 Byte')
 	if size < 10 * _kB:
@@ -463,9 +537,9 @@ def size2str(size=0, template='%s'):
 def bool2subst(boolean=None, true_return=True, false_return=False, none_return=None):
 	if boolean is None:
 		return none_return
-	if boolean is True:
+	if boolean:
 		return true_return
-	if boolean is False:
+	if not boolean:
 		return false_return
 	raise ValueError('bool2subst(): <boolean> arg must be either of True, False, None')
 #---------------------------------------------------------------------------
@@ -476,9 +550,15 @@ def bool2str(boolean=None, true_str='True', false_str='False'):
 		false_return = false_str
 	)
 #---------------------------------------------------------------------------
-def none_if(value=None, none_equivalent=None):
+def none_if(value=None, none_equivalent=None, strip_string=False):
 	"""Modelled after the SQL NULLIF function."""
-	if value == none_equivalent:
+	if value is None:
+		return None
+	if strip_string:
+		stripped = value.strip()
+	else:
+		stripped = value
+	if stripped == none_equivalent:
 		return None
 	return value
 #---------------------------------------------------------------------------
@@ -551,6 +631,9 @@ def capitalize(text=None, mode=CAPS_NAMES):
 	if (mode is None) or (mode == CAPS_NONE):
 		return text
 
+	if len(text) == 0:
+		return text
+
 	if mode == CAPS_FIRST:
 		if len(text) == 1:
 			return text[0].upper()
@@ -608,22 +691,63 @@ def input2int(initial=None, minval=None, maxval=None):
 	try:
 		int_val = int(val)
 	except (TypeError, ValueError):
+		_log.exception('int(%s) failed', val)
 		return False, val
 
 	if minval is not None:
 		if int_val < minval:
+			_log.debug('%s < min (%s)', val, minval)
 			return False, val
 	if maxval is not None:
 		if int_val > maxval:
+			_log.debug('%s > max (%s)', val, maxval)
 			return False, val
 
 	return True, int_val
 #---------------------------------------------------------------------------
+def strip_leading_empty_lines(lines=None, text=None, eol=u'\n', return_list=True):
+	if lines is None:
+		lines = text.split(eol)
+
+	while True:
+		if lines[0].strip(eol).strip() != u'':
+			break
+		lines = lines[1:]
+
+	if return_list:
+		return lines
+
+	return eol.join(lines)
+#---------------------------------------------------------------------------
+def strip_trailing_empty_lines(lines=None, text=None, eol=u'\n', return_list=True):
+	if lines is None:
+		lines = text.split(eol)
+
+	while True:
+		if lines[-1].strip(eol).strip() != u'':
+			break
+		lines = lines[:-1]
+
+	if return_list:
+		return lines
+
+	return eol.join(lines)
+#---------------------------------------------------------------------------
+def strip_empty_lines(lines=None, text=None, eol=u'\n', return_list=True):
+	return strip_trailing_empty_lines (
+		lines = strip_leading_empty_lines(lines = lines, text = text, eol = eol, return_list = True),
+		text = None,
+		eol = eol,
+		return_list = return_list
+	)
+#---------------------------------------------------------------------------
 def wrap(text=None, width=None, initial_indent=u'', subsequent_indent=u'', eol=u'\n'):
 	"""A word-wrap function that preserves existing line breaks
 	and most spaces in the text. Expects that existing line
 	breaks are posix newlines (\n).
 	"""
+	if width is None:
+		return text
 	wrapped = initial_indent + reduce (
 		lambda line, word, width=width: '%s%s%s' % (
 			line,
@@ -668,15 +792,15 @@ def unwrap(text=None, max_length=None, strip_whitespace=True, remove_empty_lines
 #---------------------------------------------------------------------------
 def xml_escape_string(text=None):
 	"""check for special XML characters and transform them"""
-
-	text = text.replace(u'&', u'&amp;')
-
-	return text
+	return xml_tools.escape(text)
 #---------------------------------------------------------------------------
-def tex_escape_string(text=None):
-	"""check for special LaTeX characters and transform them"""
+def tex_escape_string(text=None, replace_known_unicode=True):
+	"""check for special TeX characters and transform them"""
+
+	text = text.replace(u'\\', u'\\textbackslash')
+	text = text.replace(u'^', u'\\textasciicircum')
+	text = text.replace(u'~', u'\\textasciitilde')
 
-	text = text.replace(u'\\', u'$\\backslash$')
 	text = text.replace(u'{', u'\\{')
 	text = text.replace(u'}', u'\\}')
 	text = text.replace(u'%', u'\\%')
@@ -685,11 +809,17 @@ def tex_escape_string(text=None):
 	text = text.replace(u'$', u'\\$')
 	text = text.replace(u'_', u'\\_')
 
-	text = text.replace(u'^', u'\\verb#^#')
-	text = text.replace('~','\\verb#~#')
+	if replace_known_unicode:
+		# this should NOT be replaced for Xe(La)Tex
+		text = text.replace(u_euro, u'\\EUR')
 
 	return text
 #---------------------------------------------------------------------------
+def xetex_escape_string(text=None):
+	# a web search did not reveal anything else for Xe(La)Tex
+	# as opposed to LaTeX, except true unicode chars
+	return tex_escape_string(text = text, replace_known_unicode = False)
+#---------------------------------------------------------------------------
 def prompted_input(prompt=None, default=None):
 	"""Obtains entry from standard input.
 
@@ -833,6 +963,11 @@ if __name__ == '__main__':
 				else:
 					print "ERROR (conversion failed but was expected to work): >%s<, expected >%s<" % (test[0], test[2])
 	#-----------------------------------------------------------------------
+	def test_input2int():
+		print input2int(0)
+		print input2int('0')
+		print input2int(u'0', 0, 0)
+	#-----------------------------------------------------------------------
 	def test_coalesce():
 
 		import datetime as dt
@@ -1031,6 +1166,60 @@ second line\n
 	def test_md5():
 		print '%s: %s' % (sys.argv[2], file2md5(sys.argv[2]))
 	#-----------------------------------------------------------------------
+	def test_unicode():
+		print u_link_symbol * 10
+	#-----------------------------------------------------------------------
+	def test_xml_escape():
+		print xml_escape_string(u'<')
+		print xml_escape_string(u'>')
+		print xml_escape_string(u'&')
+	#-----------------------------------------------------------------------
+	def test_tex_escape():
+		tests = [u'\\', u'^', u'~', u'{', u'}', u'%',  u'&', u'#', u'$', u'_', u_euro]
+		tests.append(u'  '.join(tests))
+		for test in tests:
+			print u'%s:' % test, tex_escape_string(test)
+	#-----------------------------------------------------------------------
+	def test_gpg_decrypt():
+		fname = gpg_decrypt_file(filename = sys.argv[2], passphrase = sys.argv[3])
+		if fname is not None:
+			print "successfully decrypted:", fname
+	#-----------------------------------------------------------------------
+	def test_strip_trailing_empty_lines():
+		tests = [
+			u'one line, no embedded line breaks  ',
+			u'one line\nwith embedded\nline\nbreaks\n   '
+		]
+		for test in tests:
+			print 'as list:'
+			print strip_trailing_empty_lines(text = test, eol=u'\n', return_list = True)
+			print 'as string:'
+			print u'>>>%s<<<' % strip_trailing_empty_lines(text = test, eol=u'\n', return_list = False)
+		tests = [
+			['list', 'without', 'empty', 'trailing', 'lines'],
+			['list', 'with', 'empty', 'trailing', 'lines', '', '  ', '']
+		]
+		for test in tests:
+			print 'as list:'
+			print strip_trailing_empty_lines(lines = test, eol = u'\n', return_list = True)
+			print 'as string:'
+			print strip_trailing_empty_lines(lines = test, eol = u'\n', return_list = False)
+	#-----------------------------------------------------------------------
+	def test_fname_stem():
+		tests = [
+			r'abc.exe',
+			r'\abc.exe',
+			r'c:\abc.exe',
+			r'c:\d\abc.exe',
+			r'/home/ncq/tmp.txt',
+			r'~/tmp.txt',
+			r'./tmp.txt',
+			r'./.././tmp.txt',
+			r'tmp.txt'
+		]
+		for t in tests:
+			print "[%s] -> [%s]" % (t, fname_stem(t))
+	#-----------------------------------------------------------------------
 	#test_coalesce()
 	#test_capitalize()
 	#test_import_module()
@@ -1042,8 +1231,15 @@ second line\n
 	#test_get_unique_filename()
 	#test_size2str()
 	#test_wrap()
-	test_input2decimal()
+	#test_input2decimal()
+	#test_input2int()
 	#test_unwrap()
 	#test_md5()
+	#test_unicode()
+	#test_xml_escape()
+	#test_gpg_decrypt()
+	#test_strip_trailing_empty_lines()
+	test_fname_stem()
+	#test_tex_escape()
 
 #===========================================================================
